#if !defined( _JVizClient )
#define _JVizClient

#include "JAddObj.h"
#include "JSocket.h"
#include "JSocketInputStream.h"
#include "JSocketOutputStream.h"
#include "JThread.h"
#include "JCriticalSection.h"

class JVizClient : public JAddObj {
  protected:
    virtual void writeContent(class JOutputStream& os);
    virtual void readContent(class JDictionary& dict);
  public:
    enum { COMMAND, ENABLED };

    virtual const char* className() const;
    virtual JObject* clone() const;
    JVizClient();
    JVizClient(const JVizClient& obj);
    virtual ~JVizClient();
    virtual void reset();
    virtual void open();
    virtual void close();
    virtual void openSocket();
    virtual boolean connectSocket();
    virtual void closeSocket();
    virtual int inputType(int n);
    virtual int outputType(int n);
    virtual void access(int n, JLinkObj& link, const JDataType& data);
    virtual boolean inputAllowed(int n);
    virtual JString inputTag(int n);
    virtual JString outputTag(int n);
    virtual JFRect inputArea(int n);
    virtual JFRect outputArea(int n);
    virtual int inputFace(int n);
    virtual JFPoint inputPoint(int n, JLinkObj& link);
    virtual JFPoint outputPoint(int n, JLinkObj& link);
    virtual void paint(JGraphics g, double dx, double dy, JRegion& rgn, double scale);
    virtual void draw(JGraphics g, int x, int y, int w, int h);
    virtual void engine(int n, JLinkObj& link);
    virtual void run();
    virtual JArray getProperties();
    virtual boolean updateProperty(JProperty& prop);
    virtual void startup();
  protected:
    JSocket socket;
    JSocketInputStream sis;
    JSocketOutputStream sos;
    JString address;
	JString renderer_config;
    JString result;
	JString command;  // the incoming command

	boolean threadIsRunning;

    int port, enabled;
    uchar ssequence;
    boolean connected;
    JThread thread;
    JArray data;
    JCriticalSection cs;
    JString defList;
    boolean waitUntilBroadcasted;

    // inputs
    int x;
	int y;
	int z;
	int signal1;
	int signal2;
	int signal3;
	int signal4;
	int signal5;

    // calibration data
    int minX;
    int maxX;
    int minY;
    int maxY;
    int minZ;
    int maxZ;
    int calibrated;
    int initial_X_values_set;
    int initial_Y_values_set;
    int initial_Z_values_set;

    int minSignal1;
    int maxSignal1;
    int initial_signal1_values_set;

    int minSignal2;
    int maxSignal2;
    int initial_signal2_values_set;

    int minSignal3;
    int maxSignal3;
    int initial_signal3_values_set;

    int minSignal4;
    int maxSignal4;
    int initial_signal4_values_set;

    int minSignal5;
    int maxSignal5;
    int initial_signal5_values_set;

    // controls
    int timer_sampled;
    int calibrating;
    int sendCalibrationToViz;
};

#endif
